@@ -2,8 +2,8 @@
 #define __ORBIT_DEBUG_H__
 
 /*
- * Flip this switch to dump GIOP messages
- * as they are sent and received.
+ * Flip this switch to dump 
+ * general debug messages.
  */
 #undef DEBUG
 
@@ -31,16 +31,12 @@
 
 static inline void dprintf (const char *format, ...) { };
 #define dump_arg(a,b)
-#define do_giop_dump_send(a)
-#define do_giop_dump_recv(a)
 
 #else /* DEBUG */
 
 #include <stdio.h>
 
-#define dprintf(format...) fprintf(stderr, format)
-#define do_giop_dump_send(a) giop_dump_send(a)
-#define do_giop_dump_recv(a) giop_dump_recv(a)
+#define dprintf(...) fprintf(stderr, __VA_ARGS__)
 
 static inline void
 dump_arg (const ORBit_IArg *a, CORBA_TypeCode tc)
@@ -76,7 +72,7 @@ void     ORBit_trace_header     (CORBA_Object         object,
 				 ORBit_IMethod       *m_data);
 void     ORBit_trace_end_method (void);
 
-#define tprintf(format...) fprintf(stderr, format)
+#define tprintf(...) fprintf(stderr, __VA_ARGS__)
 #define tprintf_header(obj,md) ORBit_trace_header(obj,md)
 #define tprintf_trace_value(a,b) \
 		ORBit_trace_value ((gconstpointer *)(a), (b))
