@@ -406,6 +406,27 @@ testCompoundStruct (test_TestFactory   factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
+static void
+testObjectStruct (test_TestFactory   factory, 
+		  CORBA_Environment *ev)
+{
+  test_StructServer objref;
+  test_ObjectStruct inArg;
+
+  d_print ("Testing object structs...\n");
+  objref = test_TestFactory_getStructServer (factory, ev);
+  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+  inArg.serv = objref;
+
+  test_StructServer_opObjectStruct (objref, &inArg, ev);
+  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+  CORBA_Object_release (objref, ev);
+  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+}
+
 static void
 testStructAny (test_TestFactory   factory, 
 	       CORBA_Environment *ev)
@@ -459,21 +480,17 @@ testUnboundedSequence (test_TestFactory   factory,
   retn = test_SequenceServer_opStrSeq (objref, &inArg, &inoutArg, &outArg, ev);
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
   
-  for (i=0;i<inArg._length;i++){
+  for (i=0;i<inArg._length;i++)
 	g_assert (strcmp (inArg._buffer[i], constants_SEQ_STRING_IN[i]) == 0);
-  }
 
-  for (i=0;i<inoutArg._length;i++){
+  for (i=0;i<inoutArg._length;i++)
 	g_assert (strcmp (inoutArg._buffer[i], constants_SEQ_STRING_INOUT_OUT[i]) == 0);
-  }
 
-  for (i=0;i<outArg->_length;i++){
+  for (i=0;i<outArg->_length;i++)
 	g_assert (strcmp (outArg->_buffer[i], constants_SEQ_STRING_OUT[i]) == 0);
-  }
 
-  for (i=0;i<retn->_length;i++){
+  for (i=0;i<retn->_length;i++)
 	g_assert (strcmp (retn->_buffer[i], constants_SEQ_STRING_RETN[i]) == 0);
-  }
 
   CORBA_free (inArg._buffer);
   CORBA_free (inoutArg._buffer);
@@ -997,6 +1014,25 @@ testContext (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
+#define SEQ_SIZE 4096
+static test_StrSeq *
+make_large_str_seq (void)
+{
+	test_StrSeq *seq = test_StrSeq__alloc ();
+	int i;
+	static const char base_data[] = "This is a longish test string it could go on for ever and ever";
+
+	seq->_buffer = test_StrSeq_allocbuf (SEQ_SIZE);
+
+	for (i = 0; i < SEQ_SIZE; i++) {
+		int len = 3 + (int) ((sizeof (base_data) - 3.0) * rand () / (RAND_MAX + 1.0));
+		seq->_buffer [i] = CORBA_string_dup (base_data);
+		seq->_buffer [i] [len] = '\0';
+	}
+
+	return seq;
+}
+
 static void
 testMisc (test_TestFactory   factory, 
 	  CORBA_Environment *ev)
@@ -1015,6 +1051,7 @@ testMisc (test_TestFactory   factory,
 	}
 	
 	if (!in_proc) {
+		test_StrSeq *seq;
 		test_BasicServer objref;
 
 		objref = test_TestFactory_getBasicServer (factory, ev);
@@ -1027,6 +1064,11 @@ testMisc (test_TestFactory   factory,
 		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
 		g_assert (!strcmp (ev->_id, "IDL:CORBA/NO_IMPLEMENT:1.0"));
 		CORBA_exception_free (ev);
+		
+		seq = make_large_str_seq ();
+		test_BasicServer_testLargeStringSeq (objref, seq, ev);
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+		CORBA_free (seq);
 
 		CORBA_Object_release (objref, ev);
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
@@ -1266,6 +1308,8 @@ testPingPong (test_TestFactory   factory,
 	      CORBA_Environment *ev)
 {
 	test_PingPongServer r_objref, l_objref, objref;
+	CORBA_unsigned_long before_remote_hash;
+	CORBA_unsigned_long after_remote_hash;
 
 	d_print ("Testing ping pong invocations ...\n");
 	r_objref = test_TestFactory_createPingPongServer (factory, ev);
@@ -1276,6 +1320,8 @@ testPingPong (test_TestFactory   factory,
 	CORBA_Object_release (l_objref, ev); /* only want 1 ref */
 	g_assert (ORBit_small_get_servant (l_objref) != NULL);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+	before_remote_hash = CORBA_Object_hash (l_objref, 0, ev);
 
 	test_PingPongServer_pingPong (r_objref, l_objref, 64, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
@@ -1289,6 +1335,11 @@ testPingPong (test_TestFactory   factory,
 	g_assert (ORBit_small_get_servant (objref) != NULL);
 	g_assert (l_objref == objref);
 
+	after_remote_hash = CORBA_Object_hash (l_objref, 0, ev);
+
+	d_print ("Testing hashing\n");
+	g_assert (before_remote_hash == after_remote_hash);
+
 	d_print ("Testing equivalence\n");
 	g_assert (CORBA_Object_is_equivalent (
 		l_objref, objref, ev));
@@ -1425,6 +1476,7 @@ run_tests (test_TestFactory   factory,
 		testFixedLengthStruct (factory, ev);
 		testVariableLengthStruct (factory, ev);
 		testCompoundStruct (factory, ev);
+		testObjectStruct (factory, ev);
 		testStructAny (factory, ev);
 		testUnboundedSequence (factory, ev);
 		testBoundedSequence (factory, ev);
